@@ -11,13 +11,24 @@
 #include "Encrypt.h"
 #include "Mayaqua.h"
 
+#include <openssl/ssl.h> // This is needed only for the SSL/TLS version defines
+
 #ifdef OS_UNIX
 #include <netinet/in.h>
 
+#ifdef UNIX_LINUX
+#include <pthread.h>
+#endif
+
 #ifdef UNIX_OPENBSD
 #include <pthread.h>
 #endif
+
+#ifdef UNIX_MACOS
+#include <pthread.h>
 #endif
+#endif
+
 
 // Dynamic Value
 struct DYN_VALUE
@@ -87,7 +98,7 @@ struct IP
 
 #define	CmpIpAddr(ip1, ip2)			(Cmp((ip1)->address, (ip2)->address, sizeof((ip1)->address)))
 
-#define	IsIP6(ip)					(IsIP4(ip) == false)
+#define	IsIP6(ip)					(ip && IsIP4(ip) == false)
 #define	IsZeroIp(ip)				(IsZeroIP(ip))
 
 // IPv6 address (different format)
@@ -285,9 +296,10 @@ struct ROUTE_ENTRY
 	bool LocalRouting;
 	bool PPPConnection;
 	UINT Metric;
-	UINT OldIfMetric;
+	UINT IfMetric;
 	UINT InterfaceID;
 	UINT64 InnerScore;
+	bool Active;
 };
 
 // Routing table
@@ -412,6 +424,7 @@ struct TUBE
 	bool IsInFlushList;					// Whether it is registered in the Tube Flush List
 	void *Param1, *Param2, *Param3;
 	UINT IntParam1, IntParam2, IntParam3;
+	UINT64 DataTimeout;
 };
 
 // Data that is to send and to receive in the tube
@@ -530,6 +543,7 @@ struct SSL_PIPE
 {
 	bool ServerMode;					// Whether it's in the server mode
 	bool IsDisconnected;				// Disconnected
+	int SslVersion;
 	SSL *ssl;							// SSL object
 	struct ssl_ctx_st *ssl_ctx;			// SSL_CTX
 	SSL_BIO *SslInOut;					// I/O BIO for the data in the SSL tunnel
@@ -794,6 +808,37 @@ struct RUDP_STACK
 	USHORT Client_IcmpId, Client_IcmpSeqNo;	// Sequence number and ICMP ID that is randomly generated on the client side
 };
 
+// Data for the thread for serial connection attempts to a list of addresses
+struct CONNECT_SERIAL_PARAM
+{
+	LIST *IpList;
+	UINT LocalPort;						// Local port number to bind
+	IP   *LocalIP;						// Local IP address to bind. NULL address allowed to use.
+	IP   LocalIP_Cache;					// Local IP address to bind
+	UINT Port;
+	UINT Timeout;
+	char Hostname[MAX_SIZE];
+	char HintStr[MAX_SIZE];
+	bool No_Get_Hostname;
+	bool *CancelFlag;
+	bool *NoDelayFlag;
+	UINT *NatT_ErrorCode;
+	char NatT_SvcName[MAX_SIZE];
+	SOCK *Sock;
+	bool Finished;
+	bool Ok;
+	UINT64 FinishedTick;
+	EVENT *FinishEvent;
+	UINT Delay;
+	UINT RetryDelay;
+	bool Tcp_TryStartSsl;
+	SSL_VERIFY_OPTION *SslOption;
+	UINT *SslErr;
+	bool Use_NatT;
+	bool Force_NatT;
+	IP *Ret_Ip;
+};
+
 // Data for the thread for concurrent connection attempts for the R-UDP and TCP
 struct CONNECT_TCP_RUDP_PARAM
 {
@@ -815,11 +860,23 @@ struct CONNECT_TCP_RUDP_PARAM
 	UINT RUdpProtocol;
 	UINT Delay;
 	bool Tcp_TryStartSsl;
+	SSL_VERIFY_OPTION *SslOption;
+	UINT *SslErr;
 	LOCK *CancelLock;
 	SOCK *CancelDisconnectSock;
 	bool Tcp_InNegotiation;
 };
 
+struct SSL_VERIFY_OPTION
+{
+	bool VerifyPeer;				// Whether to verify SSL peer
+	bool PromptOnVerifyFail;		// Prompt on verification failure (Windows)
+	bool AddDefaultCA;				// Use default trust store
+	bool VerifyHostname;			// Verify server hostname
+	LIST *CaList;					// Trusted CA list
+	X *SavedCert;					// Saved server certificate
+};
+
 #define	SSL_DEFAULT_CONNECT_TIMEOUT		(15 * 1000)		// SSL default timeout
 
 // Header for TCP Pair
@@ -894,6 +951,13 @@ bool DetectIsServerSoftEtherVPN(SOCK *s);
 void ConnectThreadForTcp(THREAD *thread, void *param);
 void ConnectThreadForRUDP(THREAD *thread, void *param);
 void ConnectThreadForOverDnsOrIcmp(THREAD *thread, void *param);
+void ConnectThreadForIPv4(THREAD *thread, void *param);
+void ConnectThreadForIPv6(THREAD *thread, void *param);
+
+void BindConnectThreadForIPv4(THREAD *thread, void *param);
+void BindConnectThreadForIPv6(THREAD *thread, void *param);
+
+SOCK *CreateTCPSock(SOCKET s, bool is_ipv6, IP *current_ip, bool no_get_hostname, char *hostname_original);
 SOCK *NewRUDPClientNatT(char *svc_name, IP *ip, UINT *error_code, UINT timeout, bool *cancel, char *hint_str, char *target_hostname);
 RUDP_STACK *NewRUDPServer(char *svc_name, RUDP_STACK_INTERRUPTS_PROC *proc_interrupts, RUDP_STACK_RPC_RECV_PROC *proc_rpc_recv, void *param, UINT port, bool no_natt_register, bool over_dns_mode, volatile UINT *natt_global_udp_port, UCHAR rand_port_id, IP *listen_ip);
 SOCK *NewRUDPClientDirect(char *svc_name, IP *ip, UINT port, UINT *error_code, UINT timeout, bool *cancel, SOCK *sock, SOCK_EVENT *sock_event, UINT local_port, bool over_dns_mode);
@@ -957,14 +1021,12 @@ void Win32Select(SOCKSET *set, UINT timeout, CANCEL *c1, CANCEL *c2);
 void Win32InitAsyncSocket(SOCK *sock);
 void Win32JoinSockToSockEvent(SOCK *sock, SOCK_EVENT *event);
 void Win32FreeAsyncSocket(SOCK *sock);
-void Win32IpForwardRowToRouteEntry(ROUTE_ENTRY *entry, void *ip_forward_row);
-void Win32RouteEntryToIpForwardRow(void *ip_forward_row, ROUTE_ENTRY *entry);
+void Win32IpForwardRow2ToRouteEntry(ROUTE_ENTRY *entry, void *ip_forward_row);
+void Win32RouteEntryToIpForwardRow2(void *ip_forward_row, ROUTE_ENTRY *entry);
 int Win32CompareRouteEntryByMetric(void *p1, void *p2);
-ROUTE_TABLE *Win32GetRouteTable();
-bool Win32AddRouteEntry(ROUTE_ENTRY *e, bool *already_exists);
-void Win32DeleteRouteEntry(ROUTE_ENTRY *e);
-void Win32UINTToIP(IP *ip, UINT i);
-UINT Win32IPToUINT(IP *ip);
+ROUTE_TABLE *Win32GetRouteTable2(bool ipv4, bool ipv6);
+bool Win32AddRouteEntry2(ROUTE_ENTRY *e, bool *already_exists);
+void Win32DeleteRouteEntry2(ROUTE_ENTRY *e);
 UINT Win32GetVLanInterfaceID(char *instance_name);
 char **Win32EnumVLan(char *tag_name);
 void Win32Cancel(CANCEL *c);
@@ -976,12 +1038,11 @@ void Win32CleanupSockEvent(SOCK_EVENT *event);
 bool Win32WaitSockEvent(SOCK_EVENT *event, UINT timeout);
 bool Win32GetDefaultDns(IP *ip, char *domain, UINT size);
 bool Win32GetDnsSuffix(char *domain, UINT size);
-void Win32ReleaseDhcp9x(UINT if_id, bool wait);
 void Win32FlushDnsCache();
 int CompareIpAdapterIndexMap(void *p1, void *p2);
-ROUTE_CHANGE *Win32NewRouteChange();
-void Win32FreeRouteChange(ROUTE_CHANGE *r);
-bool Win32IsRouteChanged(ROUTE_CHANGE *r);
+ROUTE_CHANGE *Win32NewRouteChange2(bool ipv4, bool ipv6, void *callback);
+void Win32FreeRouteChange2(ROUTE_CHANGE *r);
+bool Win32IsRouteChanged2(ROUTE_CHANGE *r);
 bool Win32GetAdapterFromGuid(void *a, char *guid);
 SOCKET Win32Accept(SOCK *sock, SOCKET s, struct sockaddr *addr, int *addrlen, bool ipv6);
 
@@ -1057,7 +1118,16 @@ SOCK *ConnectEx(char *hostname, UINT port, UINT timeout);
 SOCK *ConnectEx2(char *hostname, UINT port, UINT timeout, bool *cancel_flag);
 SOCK *ConnectEx3(char *hostname, UINT port, UINT timeout, bool *cancel_flag, char *nat_t_svc_name, UINT *nat_t_error_code, bool try_start_ssl, bool no_get_hostname);
 SOCK *ConnectEx4(char *hostname, UINT port, UINT timeout, bool *cancel_flag, char *nat_t_svc_name, UINT *nat_t_error_code, bool try_start_ssl, bool no_get_hostname, IP *ret_ip);
+SOCK *ConnectEx5(char *hostname, UINT port, UINT timeout, bool *cancel_flag, char *nat_t_svc_name, UINT *nat_t_error_code, bool try_start_ssl, bool no_get_hostname, SSL_VERIFY_OPTION *ssl_option, UINT *ssl_err, char *hint_str, IP *ret_ip);
 SOCKET ConnectTimeoutIPv4(IP *ip, UINT port, UINT timeout, bool *cancel_flag);
+
+// New function named with prefix "Bind" binds outgoing connection to a specific address. New one is wrapped in original one.
+#define	BIND_LOCALIP_NULL			NULL		// NULL IP address specifies no binding
+#define	BIND_LOCALPORT_NULL			0			// NULL port number specifies no binding
+SOCK *BindConnectEx4(IP *localIP, UINT localport, char *hostname, UINT port, UINT timeout, bool *cancel_flag, char *nat_t_svc_name, UINT *nat_t_error_code, bool try_start_ssl, bool no_get_hostname, IP *ret_ip);
+SOCK *BindConnectEx5(IP *localIP, UINT localport, char *hostname, UINT port, UINT timeout, bool *cancel_flag, char *nat_t_svc_name, UINT *nat_t_error_code, bool try_start_ssl, bool no_get_hostname, SSL_VERIFY_OPTION *ssl_option, UINT *ssl_err, char *hint_str, IP *ret_ip);
+SOCKET BindConnectTimeoutIPv4(IP *localIP, UINT localport, IP *ip, UINT port, UINT timeout, bool *cancel_flag);
+
 bool SetSocketBufferSize(SOCKET s, bool send, UINT size);
 UINT SetSocketBufferSizeWithBestEffort(SOCKET s, bool send, UINT size);
 void InitUdpSocketBufferSize(SOCKET s);
@@ -1069,6 +1139,7 @@ SOCK *ListenEx(UINT port, bool local_only);
 SOCK *ListenEx2(UINT port, bool local_only, bool enable_ca, IP *listen_ip);
 SOCK *ListenEx6(UINT port, bool local_only);
 SOCK *ListenEx62(UINT port, bool local_only, bool enable_ca);
+SOCK *ListenEx63(UINT port, bool local_only, bool enable_ca, IP *listen_ip);
 SOCK *Accept(SOCK *sock);
 SOCK *Accept6(SOCK *sock);
 UINT Send(SOCK *sock, void *data, UINT size, bool secure);
@@ -1079,6 +1150,8 @@ UINT SecureSend(SOCK *sock, void *data, UINT size);
 UINT SecureRecv(SOCK *sock, void *data, UINT size);
 bool StartSSL(SOCK *sock, X *x, K *priv);
 bool StartSSLEx(SOCK *sock, X *x, K *priv, UINT ssl_timeout, char *sni_hostname);
+bool StartSSLEx2(SOCK *sock, X *x, K *priv, LIST *chain, UINT ssl_timeout, char *sni_hostname);
+bool StartSSLEx3(SOCK *sock, X *x, K *priv, LIST *chain, UINT ssl_timeout, char *sni_hostname, SSL_VERIFY_OPTION *ssl_option, UINT *ssl_err);
 bool AddChainSslCert(struct ssl_ctx_st *ctx, X *x);
 void AddChainSslCertOnDirectory(struct ssl_ctx_st *ctx);
 bool SendAll(SOCK *sock, void *data, UINT size, bool secure);
@@ -1164,6 +1237,7 @@ SOCKLIST *NewSockList();
 void StopSockList(SOCKLIST *sl);
 void FreeSockList(SOCKLIST *sl);
 bool IsIPv6Supported();
+bool HasIPv6Address();
 void SetSockTos(SOCK *s, int tos);
 void SetSockHighPriority(SOCK *s, bool flag);
 void InitIpClientList();
@@ -1223,6 +1297,7 @@ void GetLoopbackAddress6(IP *ip);
 UINT GetIPAddrType6(IP *ip);
 UINT GetIPv6AddrType(IPV6_ADDR *addr);
 void GetPrefixAddress6(IP *dst, IP *ip, IP *subnet);
+bool IsInSameNetwork(IP *a1, IP *a2, IP *subnet);
 bool IsInSameNetwork6(IP *a1, IP *a2, IP *subnet);
 bool IsInSameNetwork6ByStr(char *ip1, char *ip2, char *subnet);
 void GenerateEui64Address6(UCHAR *dst, UCHAR *mac);
@@ -1353,6 +1428,8 @@ struct SslClientCertInfo {
 
 SSL_PIPE *NewSslPipe(bool server_mode, X *x, K *k, DH_CTX *dh);
 SSL_PIPE *NewSslPipeEx(bool server_mode, X *x, K *k, DH_CTX *dh, bool verify_peer, struct SslClientCertInfo *clientcert);
+SSL_PIPE *NewSslPipeEx2(bool server_mode, X *x, K *k, LIST *chain, DH_CTX *dh, bool verify_peer, struct SslClientCertInfo *clientcert);
+SSL_PIPE* NewSslPipeEx3(bool server_mode, X* x, K* k, LIST* chain, DH_CTX* dh, bool verify_peer, struct SslClientCertInfo* clientcert, int tls13ticketscnt, bool disableTls13);
 void FreeSslPipe(SSL_PIPE *s);
 bool SyncSslPipe(SSL_PIPE *s);
 
